@@ -32,6 +32,11 @@ typedef struct
 	BuildAccumulator accum;
 }	RumBuildState;
 
+#if PG_VERSION_NUM >= 110000
+#define IndexBuildHeapScan(A, B, C, D, E, F) \
+IndexBuildHeapScan(A, B, C, D, E, F, NULL)
+#endif
+
 /*
  * Creates new posting tree with one page, containing the given TIDs.
  * Returns the page number (which will be the root of this posting tree).
@@ -93,7 +98,7 @@ createPostingTree(RumState * rumstate, OffsetNumber attnum, Relation index,
  * Form a tuple for entry tree.
  *
  * If the tuple would be too big to be stored, function throws a suitable
- * error if errorTooBig is TRUE, or returns NULL if errorTooBig is FALSE.
+ * error if errorTooBig is true, or returns NULL if errorTooBig is false.
  *
  * See src/backend/access/gin/README for a description of the index tuple
  * format that is being built here.  We build on the assumption that we
@@ -196,6 +201,8 @@ RumFormTuple(RumState * rumstate,
 	{
 		itup = repalloc(itup, newsize);
 
+		memset((char *) itup + IndexTupleSize(itup),
+			   0, newsize - IndexTupleSize(itup));
 		/* set new size in tuple header */
 		itup->t_info &= ~INDEX_SIZE_MASK;
 		itup->t_info |= newsize;
@@ -297,7 +304,7 @@ addItemPointersToLeafTuple(RumState * rumstate,
 			buildStats->nDataPages++;
 
 		/* Now insert the TIDs-to-be-added into the posting tree */
-		gdi = rumPrepareScanPostingTree(rumstate->index, postingRoot, FALSE,
+		gdi = rumPrepareScanPostingTree(rumstate->index, postingRoot, false,
 										ForwardScanDirection, attnum, rumstate);
 		rumInsertItemPointers(rumstate, attnum, gdi, items, nitem, buildStats);
 
@@ -374,7 +381,7 @@ buildFreshLeafTuple(RumState * rumstate,
 		{
 			RumPostingTreeScan *gdi;
 
-			gdi = rumPrepareScanPostingTree(rumstate->index, postingRoot, FALSE,
+			gdi = rumPrepareScanPostingTree(rumstate->index, postingRoot, false,
 											ForwardScanDirection,
 											attnum, rumstate);
 
@@ -441,7 +448,7 @@ rumEntryInsert(RumState * rumstate,
 
 			/* insert into posting tree */
 			gdi = rumPrepareScanPostingTree(rumstate->index, rootPostingTree,
-											FALSE, ForwardScanDirection,
+											false, ForwardScanDirection,
 											attnum, rumstate);
 			rumInsertItemPointers(rumstate, attnum, gdi, items,
 								  nitem, buildStats);
@@ -454,7 +461,7 @@ rumEntryInsert(RumState * rumstate,
 		itup = addItemPointersToLeafTuple(rumstate, itup,
 										  items, nitem, buildStats);
 
-		btree.isDelete = TRUE;
+		btree.isDelete = true;
 	}
 	else
 	{
@@ -633,17 +640,11 @@ rumbuild(Relation heap, Relation index, struct IndexInfo *indexInfo)
 	 * create a temporary memory context that is reset once for each tuple
 	 * inserted into the index
 	 */
-	buildstate.tmpCtx = AllocSetContextCreate(CurrentMemoryContext,
-											  "Rum build temporary context",
-											  ALLOCSET_DEFAULT_MINSIZE,
-											  ALLOCSET_DEFAULT_INITSIZE,
-											  ALLOCSET_DEFAULT_MAXSIZE);
+	buildstate.tmpCtx = RumContextCreate(CurrentMemoryContext,
+										 "Rum build temporary context");
 
-	buildstate.funcCtx = AllocSetContextCreate(CurrentMemoryContext,
-					 "Rum build temporary context for user-defined function",
-											   ALLOCSET_DEFAULT_MINSIZE,
-											   ALLOCSET_DEFAULT_INITSIZE,
-											   ALLOCSET_DEFAULT_MAXSIZE);
+	buildstate.funcCtx = RumContextCreate(CurrentMemoryContext,
+					 "Rum build temporary context for user-defined function");
 
 	buildstate.accum.rumstate = &buildstate.rumstate;
 	rumInitBA(&buildstate.accum);
@@ -813,11 +814,8 @@ ruminsert(Relation index, Datum *values, bool *isnull,
 	Datum		outerAddInfo = (Datum) 0;
 	bool		outerAddInfoIsNull = true;
 
-	insertCtx = AllocSetContextCreate(CurrentMemoryContext,
-									  "Rum insert temporary context",
-									  ALLOCSET_DEFAULT_MINSIZE,
-									  ALLOCSET_DEFAULT_INITSIZE,
-									  ALLOCSET_DEFAULT_MAXSIZE);
+	insertCtx = RumContextCreate(CurrentMemoryContext,
+								 "Rum insert temporary context");
 
 	oldCtx = MemoryContextSwitchTo(insertCtx);
 
