@@ -35,16 +35,10 @@ rumbeginscan(Relation rel, int nkeys, int norderbys)
 	so->firstCall = true;
 	so->totalentries = 0;
 	so->sortedEntries = NULL;
-	so->tempCtx = AllocSetContextCreate(CurrentMemoryContext,
-										"Rum scan temporary context",
-										ALLOCSET_DEFAULT_MINSIZE,
-										ALLOCSET_DEFAULT_INITSIZE,
-										ALLOCSET_DEFAULT_MAXSIZE);
-	so->keyCtx = AllocSetContextCreate(CurrentMemoryContext,
-									   "Rum scan key context",
-									   ALLOCSET_DEFAULT_MINSIZE,
-									   ALLOCSET_DEFAULT_INITSIZE,
-									   ALLOCSET_DEFAULT_MAXSIZE);
+	so->tempCtx = RumContextCreate(CurrentMemoryContext,
+								   "Rum scan temporary context");
+	so->keyCtx = RumContextCreate(CurrentMemoryContext,
+								  "Rum scan key context");
 
 	initRumState(&so->rumstate, scan->indexRelation);
 
@@ -383,7 +377,9 @@ initScanKey(RumScanOpaque so, ScanKey skey, bool *hasPartialMatch)
 	 */
 	if (skey->sk_flags & SK_ISNULL)
 	{
-		so->isVoidRes = true;
+		/* Do not set isVoidRes for order keys */
+		if ((skey->sk_flags & SK_ORDER_BY) == 0)
+			so->isVoidRes = true;
 		return;
 	}
 
@@ -415,7 +411,9 @@ initScanKey(RumScanOpaque so, ScanKey skey, bool *hasPartialMatch)
 	{
 		if (searchMode == GIN_SEARCH_MODE_DEFAULT)
 		{
-			so->isVoidRes = true;
+			/* Do not set isVoidRes for order keys */
+			if ((skey->sk_flags & SK_ORDER_BY) == 0)
+				so->isVoidRes = true;
 			return;
 		}
 		nQueryValues = 0;		/* ensure sane value */
@@ -612,11 +610,7 @@ rumNewScanKey(IndexScanDesc scan)
 	}
 
 	for (i = 0; i < scan->numberOfOrderBys; i++)
-	{
 		initScanKey(so, &scan->orderByData[i], NULL);
-		if (so->isVoidRes)
-			break;
-	}
 
 	/*
 	 * Fill markAddInfo if possible
