@@ -20,14 +20,15 @@
 
 #include "rum.h"
 
+/*
+ * A "wrapper" over tsquery item.  More suitable representation for pocessing.
+ */
 typedef struct QueryItemWrap
 {
 	QueryItemType type;
 	int8		oper;
 	bool		not;
-	int			operandsCount,
-				operandsAllocated;
-	struct QueryItemWrap *operands;
+	List	   *operands;
 	struct QueryItemWrap *parent;
 	int			distance,
 				length;
@@ -35,40 +36,29 @@ typedef struct QueryItemWrap
 	int			num;
 }	QueryItemWrap;
 
+/*
+ * Add child to tsquery item wrap.
+ */
 static QueryItemWrap *
-add_child(QueryItemWrap * parent)
+add_child(QueryItemWrap *parent)
 {
 	QueryItemWrap *result;
 
-	if (!parent)
+	result = (QueryItemWrap *) palloc0(sizeof(QueryItemWrap));
+
+	if (parent)
 	{
-		result = (QueryItemWrap *) palloc0(sizeof(QueryItemWrap));
-	}
-	else
-	{
-		parent->operandsCount++;
-		while (parent->operandsCount > parent->operandsAllocated)
-		{
-			if (parent->operandsAllocated > 0)
-			{
-				parent->operandsAllocated *= 2;
-				parent->operands = (QueryItemWrap *) repalloc(parent->operands, parent->operandsAllocated * sizeof(*parent->operands));
-			}
-			else
-			{
-				parent->operandsAllocated = 4;
-				parent->operands = (QueryItemWrap *) palloc(parent->operandsAllocated * sizeof(*parent->operands));
-			}
-		}
-		result = &parent->operands[parent->operandsCount - 1];
-		memset(result, 0, sizeof(*result));
 		result->parent = parent;
+		parent->operands = lappend(parent->operands, result);
 	}
 	return result;
 }
 
+/*
+ * Make wrapper over tsquery item.  Flattern tree if needed.
+ */
 static QueryItemWrap *
-make_query_item_wrap(QueryItem *item, QueryItemWrap * parent, bool not)
+make_query_item_wrap(QueryItem *item, QueryItemWrap *parent, bool not)
 {
 	if (item->type == QI_VAL)
 	{
@@ -126,16 +116,21 @@ make_query_item_wrap(QueryItem *item, QueryItemWrap * parent, bool not)
 	return NULL;
 }
 
+/*
+ * Recursively calculate "sum" for tsquery item wraps.
+ */
 static int
-calc_wraps(QueryItemWrap * wrap, int *num)
+calc_wraps(QueryItemWrap *wrap, int *num)
 {
-	int			i,
-				notCount = 0,
+	int			notCount = 0,
 				result;
+	ListCell   *lc;
 
-	for (i = 0; i < wrap->operandsCount; i++)
+	foreach(lc, wrap->operands)
 	{
-		if (wrap->operands[i].not)
+		QueryItemWrap *item = (QueryItemWrap *) lfirst(lc);
+
+		if (item->not)
 			notCount++;
 	}
 
@@ -143,7 +138,7 @@ calc_wraps(QueryItemWrap * wrap, int *num)
 	{
 		wrap->num = (*num)++;
 		if (wrap->oper == OP_AND)
-			wrap->sum = notCount + 1 - wrap->operandsCount;
+			wrap->sum = notCount + 1 - list_length(wrap->operands);
 		if (wrap->oper == OP_OR)
 			wrap->sum = notCount;
 	}
@@ -153,11 +148,19 @@ calc_wraps(QueryItemWrap * wrap, int *num)
 	}
 
 	result = 0;
-	for (i = 0; i < wrap->operandsCount; i++)
-		result += calc_wraps(&wrap->operands[i], num);
+	foreach(lc, wrap->operands)
+	{
+		QueryItemWrap *item = (QueryItemWrap *) lfirst(lc);
+
+		result += calc_wraps(item, num);
+	}
 	return result;
 }
 
+/*
+ * Check if tsquery doesn't need any positive lexeme occurence for satisfaction.
+ * That is this funciton returns true when tsquery maches empty tsvector.
+ */
 static bool
 check_allnegative(QueryItemWrap * wrap)
 {
@@ -167,22 +170,26 @@ check_allnegative(QueryItemWrap * wrap)
 	}
 	else if (wrap->oper == OP_AND)
 	{
-		int			i;
+		ListCell   *lc;
 
-		for (i = 0; i < wrap->operandsCount; i++)
+		foreach(lc, wrap->operands)
 		{
-			if (!check_allnegative(&wrap->operands[i]))
+			QueryItemWrap *item = (QueryItemWrap *) lfirst(lc);
+
+			if (!check_allnegative(item))
 				return false;
 		}
 		return true;
 	}
 	else if (wrap->oper == OP_OR)
 	{
-		int			i;
+		ListCell   *lc;
 
-		for (i = 0; i < wrap->operandsCount; i++)
+		foreach(lc, wrap->operands)
 		{
-			if (check_allnegative(&wrap->operands[i]))
+			QueryItemWrap *item = (QueryItemWrap *) lfirst(lc);
+
+			if (check_allnegative(item))
 				return true;
 		}
 		return false;
@@ -195,6 +202,7 @@ check_allnegative(QueryItemWrap * wrap)
 
 }
 
+/* Max length of variable-length encoded 32-bit integer */
 #define MAX_ENCODED_LEN 5
 
 /*
@@ -262,8 +270,11 @@ typedef struct
 	char	   *operand;
 }	ExtractContext;
 
+/*
+ * Recursively extract entries from tsquery wraps.  Encode paths into addInfos.
+ */
 static void
-extract_wraps(QueryItemWrap * wrap, ExtractContext * context, int level)
+extract_wraps(QueryItemWrap *wrap, ExtractContext *context, int level)
 {
 	if (wrap->type == QI_VAL)
 	{
@@ -271,7 +282,7 @@ extract_wraps(QueryItemWrap * wrap, ExtractContext * context, int level)
 		unsigned char *ptr;
 		int			index;
 
-
+		/* Check if given lexeme was already extracted */
 		for (index = 0; index < context->index; index++)
 		{
 			text	   *entry;
@@ -282,6 +293,7 @@ extract_wraps(QueryItemWrap * wrap, ExtractContext * context, int level)
 				break;
 		}
 
+		/* Either allocate new addInfo or extend existing addInfo */
 		if (index >= context->index)
 		{
 			index = context->index;
@@ -291,11 +303,6 @@ extract_wraps(QueryItemWrap * wrap, ExtractContext * context, int level)
 			context->addInfo[index] = PointerGetDatum(addinfo);
 			context->addInfoIsNull[index] = false;
 			context->index++;
-
-			/*
-			 * ptrEnd = (unsigned char *) VARDATA(addinfo) + VARHDRSZ + 2 *
-			 * Max(level, 1) * MAX_ENCODED_LEN;
-			 */
 		}
 		else
 		{
@@ -304,25 +311,14 @@ extract_wraps(QueryItemWrap * wrap, ExtractContext * context, int level)
 					 VARSIZE(addinfo) + 2 * Max(level, 1) * MAX_ENCODED_LEN);
 			context->addInfo[index] = PointerGetDatum(addinfo);
 			ptr = (unsigned char *) VARDATA(addinfo) + VARSIZE_ANY_EXHDR(addinfo);
-
-			/*
-			 * ptrEnd = (unsigned char *) VARDATA(addinfo) +
-			 * VARSIZE_ANY_EXHDR(addinfo) + 2 * Max(level, 1) *
-			 * MAX_ENCODED_LEN;
-			 */
 		}
 
-		/*
-		 * elog(NOTICE, "%s",
-		 * text_to_cstring(DatumGetTextP(context->entries[index])));
-		 */
-
+		/* Encode path into addInfo */
 		while (wrap->parent)
 		{
 			QueryItemWrap *parent = wrap->parent;
 			uint32		sum;
 
-			/* elog(NOTICE, "%d %d %d", parent->num, parent->sum, wrap->not); */
 			encode_varbyte((uint32) parent->num, &ptr);
 			sum = (uint32) abs(parent->sum);
 			sum <<= 2;
@@ -338,38 +334,21 @@ extract_wraps(QueryItemWrap * wrap, ExtractContext * context, int level)
 			encode_varbyte(1, &ptr);
 			encode_varbyte(4 | 1, &ptr);
 		}
-		/* Assert(ptr <= ptrEnd); */
 		SET_VARSIZE(addinfo, ptr - (unsigned char *) addinfo);
-
-		/*
-		 * elog(NOTICE, "%s", DatumGetPointer(DirectFunctionCall1(byteaout,
-		 * PointerGetDatum(addinfo))));
-		 */
 	}
 	else if (wrap->type == QI_OPR)
 	{
-		int			i;
+		ListCell   *lc;
 
-		for (i = 0; i < wrap->operandsCount; i++)
-			extract_wraps(&wrap->operands[i], context, level + 1);
+		foreach(lc, wrap->operands)
+		{
+			QueryItemWrap *item = (QueryItemWrap *) lfirst(lc);
+
+			extract_wraps(item, context, level + 1);
+		}
 	}
 }
 
-/*PG_FUNCTION_INFO_V1(rum_process_tsquery);
-Datum
-rum_process_tsquery(PG_FUNCTION_ARGS)
-{
-	TSQuery		query = PG_GETARG_TSQUERY(0);
-	QueryItem  *item = GETQUERY(query);
-	QueryItemWrap *wrap = make_query_item_wrap(item, NULL, false);
-	int			num = 1;
-
-	calc_wraps(wrap, &num);
-	print_wraps(wrap, , 0);
-
-	PG_RETURN_VOID();
-}*/
-
 PG_FUNCTION_INFO_V1(ruminv_extract_tsquery);
 Datum
 ruminv_extract_tsquery(PG_FUNCTION_ARGS)
@@ -419,12 +398,6 @@ ruminv_extract_tsquery(PG_FUNCTION_ARGS)
 	}
 	*nentries = count;
 
-/*	elog(NOTICE, "%d", *nentries);
-	for (i = 0; i < *nentries; i++)
-	{
-		elog(NOTICE, "%s", text_to_cstring(DatumGetPointer((entries)[i])));
-	}*/
-
 	PG_FREE_IF_COPY(query, 0);
 	PG_RETURN_POINTER(entries);
 }
@@ -520,14 +493,10 @@ ruminv_tsvector_consistent(PG_FUNCTION_ARGS)
 		if (addInfoIsNull[i])
 			elog(ERROR, "Unexpected addInfoIsNull");
 
+		/* Iterate path making corresponding calculation */
 		ptr = (unsigned char *) VARDATA_ANY(DatumGetPointer(addInfo[i]));
 		size = VARSIZE_ANY_EXHDR(DatumGetPointer(addInfo[i]));
 
-		/*
-		 * elog(NOTICE, "%d %s", i,
-		 * DatumGetPointer(DirectFunctionCall1(byteaout, addInfo[i])));
-		 */
-
 		if (size == 0)
 		{
 			res = true;
@@ -549,8 +518,6 @@ ruminv_tsvector_consistent(PG_FUNCTION_ARGS)
 
 			index = num - 1;
 
-			/* elog(NOTICE, "a %d %d %d %d", i, index, sum, not); */
-
 			if (child)
 			{
 				child->parent = index;
@@ -584,17 +551,13 @@ ruminv_tsvector_consistent(PG_FUNCTION_ARGS)
 		}
 	}
 
+	/* Iterate over nodes */
 	if (allFalse && check[nkeys - 1])
 	{
 		res = true;
 	}
 	else
 	{
-		/*
-		 * for (i = 0; i < lastIndex; i++) { elog(NOTICE, "s %d %d %d %d", i,
-		 * nodes[i].sum, nodes[i].parent, nodes[i].not); }
-		 */
-
 		for (i = lastIndex - 1; i >= 0; i--)
 		{
 			if (nodes[i].parent != -2)
@@ -617,8 +580,6 @@ ruminv_tsvector_consistent(PG_FUNCTION_ARGS)
 		}
 	}
 
-/*	elog(NOTICE, "%d", res);*/
-
 	PG_RETURN_BOOL(res);
 }
 
